@@ -36,6 +36,7 @@ import (
 	"google.golang.org/grpc/internal/stubserver"
 	"google.golang.org/grpc/internal/testutils/pickfirst"
 	"google.golang.org/grpc/internal/testutils/xds/e2e"
+	"google.golang.org/grpc/internal/xds/bootstrap"
 	"google.golang.org/grpc/peer"
 	"google.golang.org/grpc/resolver"
 	"google.golang.org/grpc/resolver/manual"
@@ -112,7 +113,9 @@ func (s) TestAggregateCluster_WithTwoEDSClusters(t *testing.T) {
 				return nil
 			}
 			if len(req.GetResourceNames()) == 0 {
-				// This is the case for ACKs. Do nothing here.
+				// This happens at the end of the test when the grpc channel is
+				// being shut down and it is no longer interested in xDS
+				// resources.
 				return nil
 			}
 			select {
@@ -489,11 +492,15 @@ func (s) TestAggregateCluster_WithEDSAndDNS(t *testing.T) {
 			if req.GetTypeUrl() != version.V3EndpointsURL {
 				return nil
 			}
-			if len(req.GetResourceNames()) > 0 {
-				select {
-				case edsResourceCh <- req.GetResourceNames()[0]:
-				default:
-				}
+			if len(req.GetResourceNames()) == 0 {
+				// This happens at the end of the test when the grpc channel is
+				// being shut down and it is no longer interested in xDS
+				// resources.
+				return nil
+			}
+			select {
+			case edsResourceCh <- req.GetResourceNames()[0]:
+			default:
 			}
 			return nil
 		},
@@ -761,7 +768,7 @@ func (s) TestAggregateCluster_BadEDS_GoodToBadDNS(t *testing.T) {
 
 	// Push an error from the DNS resolver as well.
 	dnsErr := fmt.Errorf("DNS error")
-	dnsR.ReportError(dnsErr)
+	dnsR.CC().ReportError(dnsErr)
 
 	// Ensure that RPCs continue to succeed for the next second.
 	for end := time.Now().Add(time.Second); time.Now().Before(end); <-time.After(defaultTestShortTimeout) {
@@ -895,7 +902,7 @@ func (s) TestAggregateCluster_BadDNS_GoodEDS(t *testing.T) {
 
 	// Produce a bad resolver update from the DNS resolver.
 	dnsErr := fmt.Errorf("DNS error")
-	dnsR.ReportError(dnsErr)
+	dnsR.CC().ReportError(dnsErr)
 
 	// RPCs should work, higher level DNS cluster errors so should fallback to
 	// EDS cluster.
@@ -967,7 +974,7 @@ func (s) TestAggregateCluster_BadEDS_BadDNS(t *testing.T) {
 	}
 
 	// Produce a bad resolver update from the DNS resolver.
-	dnsR.ReportError(fmt.Errorf("DNS error"))
+	dnsR.CC().ReportError(fmt.Errorf("DNS error"))
 
 	// Ensure that the error from the DNS Resolver leads to an empty address
 	// update for both priorities.
@@ -1174,9 +1181,13 @@ func (s) TestAggregateCluster_Fallback_EDS_ResourceNotFound(t *testing.T) {
 
 	// Create an xDS client talking to the above management server, configured
 	// with a short watch expiry timeout.
-	xdsClient, close, err := xdsclient.NewForTesting(xdsclient.OptionsForTesting{
+	config, err := bootstrap.NewConfigFromContents(bootstrapContents)
+	if err != nil {
+		t.Fatalf("Failed to parse bootstrap contents: %s, %v", string(bootstrapContents), err)
+	}
+	pool := xdsclient.NewPool(config)
+	xdsClient, close, err := pool.NewClientForTesting(xdsclient.OptionsForTesting{
 		Name:               t.Name(),
-		Contents:           bootstrapContents,
 		WatchExpiryTimeout: defaultTestWatchExpiryTimeout,
 	})
 	if err != nil {
