@@ -55,6 +55,7 @@ import (
 	featuregatetesting "k8s.io/component-base/featuregate/testing"
 	"k8s.io/kube-openapi/pkg/validation/spec"
 	"k8s.io/kube-openapi/pkg/validation/strfmt"
+	"k8s.io/utils/ptr"
 )
 
 var stringSchema *apiextensionsv1.JSONSchemaProps = &apiextensionsv1.JSONSchemaProps{
@@ -1115,7 +1116,7 @@ func TestRatchetingFunctionality(t *testing.T) {
 					Properties: map[string]apiextensionsv1.JSONSchemaProps{
 						"field": {
 							Type:         "array",
-							XListType:    ptr("map"),
+							XListType:    ptr.To("map"),
 							XListMapKeys: []string{"name", "port"},
 							Items: &apiextensionsv1.JSONSchemaPropsOrArray{
 								Schema: &apiextensionsv1.JSONSchemaProps{
@@ -1395,7 +1396,7 @@ func TestRatchetingFunctionality(t *testing.T) {
 								{
 									Rule:            "!oldSelf.hasValue()",
 									Message:         "oldSelf must be null",
-									OptionalOldSelf: ptr(true),
+									OptionalOldSelf: ptr.To(true),
 								},
 							},
 						},
@@ -1445,7 +1446,7 @@ func TestRatchetingFunctionality(t *testing.T) {
 			Operations: []ratchetingTestOperation{
 				updateMyCRDV1Beta1Schema{&apiextensionsv1.JSONSchemaProps{
 					Type:                   "object",
-					XPreserveUnknownFields: ptr(true),
+					XPreserveUnknownFields: ptr.To(true),
 				}},
 				applyPatchOperation{
 					"create instance with strings that do not start with k8s",
@@ -1457,7 +1458,7 @@ func TestRatchetingFunctionality(t *testing.T) {
 				},
 				updateMyCRDV1Beta1Schema{&apiextensionsv1.JSONSchemaProps{
 					Type:                   "object",
-					XPreserveUnknownFields: ptr(true),
+					XPreserveUnknownFields: ptr.To(true),
 					Properties: map[string]apiextensionsv1.JSONSchemaProps{
 						"myStringField": {
 							Type: "string",
@@ -1759,10 +1760,6 @@ func TestRatchetingFunctionality(t *testing.T) {
 	runTests(t, cases)
 }
 
-func ptr[T any](v T) *T {
-	return &v
-}
-
 type validator func(new, old *unstructured.Unstructured)
 
 func newValidator(customResourceValidation *apiextensionsinternal.JSONSchemaProps, kind schema.GroupVersionKind, namespaceScoped bool) (validator, error) {
@@ -2031,7 +2028,7 @@ func TestRatchetingDropFields(t *testing.T) {
 											{
 												// Results in error if field wasn't dropped
 												Rule:            "self == oldSelf",
-												OptionalOldSelf: ptr(true),
+												OptionalOldSelf: ptr.To(true),
 											},
 										},
 									},
@@ -2064,7 +2061,7 @@ func TestRatchetingDropFields(t *testing.T) {
 		if err != nil {
 			return false, err
 		}
-		existing.Spec.Versions[0].Schema.OpenAPIV3Schema.Properties["spec"].Properties["field"].XValidations[0].OptionalOldSelf = ptr(true)
+		existing.Spec.Versions[0].Schema.OpenAPIV3Schema.Properties["spec"].Properties["field"].XValidations[0].OptionalOldSelf = ptr.To(true)
 		updated, err = apiExtensionClient.ApiextensionsV1().CustomResourceDefinitions().Update(context.TODO(), existing, metav1.UpdateOptions{})
 		if err != nil {
 			if apierrors.IsConflict(err) {
